import { getArgumentValues, isInterfaceType, isObjectType, isUnionType, Kind, parse, print, } from 'graphql';
/**
 * Extract Object types with @sourceFrom directives.
 *
 * Returns a map of type names to source package and function name tuples.
 */
export function extractSourceMapping(schema) {
    const sources = {};
    const sourceFrom = schema.getDirective('sourceFrom');
    if (!sourceFrom) {
        return sources;
    }
    for (const type of Object.values(schema.getTypeMap())) {
        if (isObjectType(type)) {
            for (const directive of type.astNode?.directives || []) {
                if (directive.name.value === 'sourceFrom') {
                    const values = getArgumentValues(sourceFrom, directive);
                    sources[type.name] = values.fn;
                    values['fn'];
                }
            }
        }
    }
    return sources;
}
/**
 * Extract a list of all union types.
 */
export function extractUnions(schema) {
    return Object.values(schema.getTypeMap())
        .filter(isUnionType)
        .map((type) => type.name);
}
/**
 * Extract a list of all interface types.
 */
export function extractInterfaces(schema) {
    return Object.values(schema.getTypeMap())
        .filter(isInterfaceType)
        .map((type) => type.name);
}
/**
 * Clean up a schema string and remove everything that could confuse Gatsby.
 */
export function cleanSchema(schema) {
    const ast = parse(schema);
    const result = [];
    ast.definitions.forEach((def) => {
        if (def.kind === Kind.SCHEMA_DEFINITION ||
            def.kind === Kind.DIRECTIVE_DEFINITION) {
            return;
        }
        result.push(print(def));
    });
    return result.join('\n');
}
